/*
 * KOHEXAMPLE : train a Kohonen network to learn clusters
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <sprlib.h>

#define eval_koh_net eval_ff_net

# define sizefield 3
# define sizeformat 128


extern void exit();
extern double atof();

main()
{
	NET *net;
	int *NetVect;
	double  alpha = .9, radius = 6.0;
	long options = (NO_HISTU | NO_HISTT );
	DATASET *learning_set;
	SAMPLE* sample_ptr;
	char**output;
	char outformat[sizeformat];
	char *blanktext;
	int sizex,sizey,nsample;
	long int totalnum;

	long cycle = 0L;
	UNIT* best_unit;
	int positionx,positiony;
	int i,j;


	printf("KOC : train a Kohonen network to learn clusters");

	sprinit(INIT_EXIT);

	/*
	 * Load learning set, and normalize it
	 */

	learning_set = load_dataset("ejobs.dat");
	normalize_input_dataset(learning_set);

	NetVect = ivector(1, 2);
	sizex = NetVect[1] = 14;
	sizey = NetVect[2] = 14;
	totalnum=sizex*sizey;

	output=(char **)malloc(totalnum*sizeof(char*));
	blanktext=(char*)malloc((sizefield+1)*sizeof(char));
	strnset(blanktext,32,sizefield);
	*(blanktext+sizefield)=0;
	sprintf(outformat,"%s%d%s","%.",sizefield,"s ");

	printf("Creating network.\n");
	if ((net = create_koh_net(1L, 9, 2, NetVect))== NULL) {
		sprerror("Problem making a feed forward network.");
		exit(1);
	}

	printf("Randomizing the network.\n");
	unif_rand_net(100, -2.0, 2.0, net);

	while (cycle <= 15000L) {
		cycle++;
		koh_learn(net,learning_set,alpha,radius,1L,options);
		if( cycle % 100 == 0){
			for(j=0;j<sizey;j++)
				for(i=0;i<sizex;i++)
				       *(output+j*sizey+i)=blanktext;
			nsample=0;
			sample_ptr=learning_set->FirstSample;
			while(sample_ptr){
				eval_koh_net(net,sample_ptr->Input);
				best_unit=best_matching_unit(net);
				sscanf(best_unit->UnitName,"Standard Kohonen processing unit at location (%d,%d)",&positionx,&positiony);
				*(output+positiony*sizey+positionx)=learning_set->OutputDescr[nsample];
				sample_ptr = sample_ptr->Next;
				nsample++;
			}

			for(j=0;j<sizey;j++){
				for(i=0;i<sizex;i++)
					printf(outformat,*(output+j*sizey+i));
				printf("\n");
			}

			
			printf("\ncycle=%ld radius=%f\n",cycle,radius);
			alpha  = MAX (alpha  * 0.999, 0.001);
			radius = MAX (radius * 0.99,  1);
		}
	}

	getch();
	delete_net(net);
	free_ivector(NetVect, 1, 2);
	free_dataset(learning_set);
	free_system_netlib();
	free_system_dataset();
	free(output);
	free(blanktext);
	sprexit(EXIT_SILENT);
	return 0;
}

/*
 * End of Source
 */
